//Copyright Plasmatronics pty ltd 2023-2024 +61394869902
import EventEmitter from './eventemitter.js';
import Format from './format.js';
import Properties from './properties.js';

export default class Model extends EventEmitter {
  constructor() {
    super();
    // this.baseApi = 'http://192.168.8.246/api/';
    // this.baseApi = 'http://192.168.75.100/api/';
    // this.baseApi = 'http://dnet-578ab8.local/api/';
    this.baseApi = 'api/';
    this.config_url = this.baseApi + 'config';
    this.history_url = this.baseApi + 'history';
    this.status_url = this.baseApi + 'status';
    this.update_url = this.baseApi + 'update';

    this.requestPending = false;

    // event handlers
    let classInstance = this;
    this.on('dingoGetStatus',
            function(ev) { classInstance.getDingoStatus(ev) });
    this.on('dingoGetHistory',
            function(ev) { classInstance.getDingoHistory(ev) });
    this.on('dingoSetConfig',
            function(ev) { classInstance.setDingoConfig(ev) });
    this.on('dingoUpload', function(ev) { classInstance.dingoUpload(ev) });
  }

  isRequestPending() { return this.requestPending }

  dingoUpload(fileName) {
    var classInstance = this;
    if (this.requestPending)
      return;
    this.requestPending = true;
    let formData = new FormData()
    formData.append('file', fileName);
    let request = new XMLHttpRequest();
    request.upload.addEventListener('progress', function(ev) {
      let progress = ((ev.loaded / ev.total) * 100).toFixed();
      classInstance.emit('dingoUploadProgress', progress.toString() + " Percent");
    });
    request.upload.addEventListener('load', function(ev) {
      let message = ev.total.toString() + " Bytes Successfully Uploaded, Hard Reset Required";
      classInstance.emit('dingoUploadDone', message);
      classInstance.requestPending = false;
    });
    request.upload.addEventListener("error", function(ev) {
      classInstance.emit('dingoUploadDone', "Upload Error");
      classInstance.requestPending = false;
    });
    request.upload.addEventListener("abort", function(ev) {
      classInstance.emit('dingoUploadDone', "Upload Aborted");
      classInstance.requestPending = false;
    });
    request.open('POST', this.update_url);
    request.send(formData);
  }

  fetchWithTimeoutGet(url, timeout) {
    return new Promise((resolve, reject) => {
      const controller = new AbortController();
      const signal = controller.signal;
      // Set timeout timer
      let timer = setTimeout(() => {
        controller.abort();
        reject(new Error('Fetch timeout'));
      }, timeout);
      // Start fetch
      fetch(url, {signal})
          .then(response => {
            if (!response.ok) {
              reject(new Error("Fetch error " + response.status));
            } else
              return response;
          })
          .then(response => response.json())
          .then(response => resolve(response))
          .catch(error => reject(error))
          .finally(() => clearTimeout(timer))
    })
  }

  fetchWithTimeoutPost(url, timeout, postData) {
    return new Promise((resolve, reject) => {
      const controller = new AbortController();
      const signal = controller.signal;
      // Set timeout timer
      let timer = setTimeout(() => {
        controller.abort();
        reject(new Error('Fetch timeout'));
      }, timeout);
      // Start fetch
      const options = {
        signal,
        method : 'POST',
        body : JSON.stringify(postData),
        headers : {'Content-Type' : 'application/json'}
      };

      fetch(url, options)
          .then(response => {
            if (!response.ok) {
              reject(new Error("Fetch error " + response.status));
            } else
              return response;
          })
          .then(response => response.json())
          .then(response => resolve(response))
          .catch(error => reject(error))
          .finally(() => clearTimeout(timer))
    })
  }

  formatData(data) {
    let result = [];
    Object.keys(data).forEach(function(key) {
      let property = Properties.find(key);
      if (typeof property != 'undefined') {
        result.push(Format.format(property, data[key]));
      } else {
        console.log("Cant find " + key);
      }
    })
    return result;
  }

  addCalculatedStatus(dingoStatus) {
    if ((dingoStatus['cint'] != null) && (dingoStatus['cext'] != null)) {
      dingoStatus['CTOT'] = dingoStatus['cint'] + dingoStatus['cext'];
    } else {
      dingoStatus['CTOT'] = null;
    }
    if ((dingoStatus['lint'] != null) && (dingoStatus['lext'] != null)) {
      dingoStatus['LTOT'] = dingoStatus['lint'] + dingoStatus['lext'];
    } else {
      dingoStatus['LTOT'] = null;
    }
    if ((dingoStatus['c-in'] != null) && (dingoStatus['ciah'] != null)) {
      dingoStatus['IEXT'] = dingoStatus['c-in'] + dingoStatus['ciah'];
    } else {
      dingoStatus['IEXT'] = null;
    }
    if ((dingoStatus['cout'] != null) && (dingoStatus['liah'] != null)) {
      dingoStatus['OEXT'] = dingoStatus['cout'] + dingoStatus['liah'];
    } else {
      dingoStatus['OEXT'] = null;
    }
    if ((dingoStatus['cfgtmr'] != null) && (dingoStatus['cfgtmr'] > 0)) {
      dingoStatus['update'] = true;
    } else {
      dingoStatus['update'] = false;
    }
    return dingoStatus;
  }

  async getDingoStatus() {
    if (this.requestPending)
      return;
    this.requestPending = true;
    let status;
    try {
      status = await this.fetchWithTimeoutGet(this.status_url, 5000);
    } catch (error) {
      this.emit('dingoGetStatusDone', null);
      this.requestPending = false;
      return;
    }
    if (status != null) {
      status = this.addCalculatedStatus(status);
      status = this.formatData(status);
    }
    this.emit('dingoGetStatusDone', status);
    this.requestPending = false;
  }

  async getDingoHistory() {
    if (this.requestPending)
      return;
    this.requestPending = true;
    let history;
    try {
      history = await this.fetchWithTimeoutGet(this.history_url, 5000);
    } catch (error) {
      this.emit('dingoGetHistoryDone', null);
      this.requestPending = false;
      return;
    }
    let table = [];
    for (let i = 0; i < 100; i++) {
      let day = {};
      Object.keys(history).forEach(
          (property) => { day[property] = history[property][i]; });
      table.push(this.formatData(day));
    }
    this.emit('dingoGetHistoryDone', table);
    this.requestPending = false;
  }

  async setDingoConfig(postData) {
    console.log(postData)
    if (this.requestPending)
      return;
    this.requestPending = true;
    let dingoConfig;
    try {
      dingoConfig =
          await this.fetchWithTimeoutPost(this.config_url, 5000, postData)
              .catch(response => console.log(response))
    } catch (error) {
      this.emit('dingoSetConfigDone', null);
      this.requestPending = false;
      console.log(error)
      return;
    }
    
    // console.log(Object.keys(dingoConfig))
    // console.log(Object.keys(postData))

    let dingoConfigKeys = Object.keys(dingoConfig);
    let postDataKeys = Object.keys(postData);
    if(dingoConfigKeys.length == 1 && postDataKeys.length == 1){
      if(dingoConfig[dingoConfigKeys[0]] == postData[postDataKeys[0]]){
        this.emit('dingoSetConfigDone', dingoConfig);
      } else {
        this.emit('dingoSetConfigDone', null);
      }
    } else {
      this.emit('dingoSetConfigDone', null);
    }

    // if (postData == dingoConfig){
    //   this.emit('dingoSetConfigDone', dingoConfig);
    // } else {
    //   this.emit('dingoSetConfigDone', null);
    // }

    this.requestPending = false;
  }
}
